# (c) Copyright 2014-2016 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import logging
import pprint

from cliff import command
from cliff import lister
from cliff import show

from freezerclient import exceptions
from freezerclient import utils


logging = logging.getLogger(__name__)


class BackupShow(show.ShowOne):
    """Show the metadata of a single backup"""
    def get_parser(self, prog_name):
        parser = super(BackupShow, self).get_parser(prog_name)
        parser.add_argument(dest='backup_uuid',
                            help='ID of the backup')
        return parser

    def take_action(self, parsed_args):
        backup = self.app.client.backups.get(parsed_args.backup_uuid)
        if not backup:
            raise exceptions.ApiClientException('Backup not found')

        column = (
            'Backup ID',
            'Metadata'
        )
        data = (
            backup.get('backup_uuid'),
            pprint.pformat(backup.get('backup_metadata'))
        )
        return column, data


class BackupList(lister.Lister):
    """List all backups for your user"""
    def get_parser(self, prog_name):
        parser = super(BackupList, self).get_parser(prog_name)

        parser.add_argument(
            '--limit',
            dest='limit',
            default=100,
            help='Specify a limit for search query',
        )

        parser.add_argument(
            '--offset',
            dest='offset',
            default=0,
            help='',
        )

        parser.add_argument(
            '--search',
            dest='search',
            default='',
            help='Define a filter for the query',
        )
        return parser

    def take_action(self, parsed_args):
        search = utils.prepare_search(parsed_args.search)

        backups = self.app.client.backups.list(limit=parsed_args.limit,
                                               offset=parsed_args.offset,
                                               search=search)

        columns = ('Backup ID', 'Backup UUID', 'Hostname', 'Path',
                   'Created at', 'Level')

        # Print empty table if no backups found
        if not backups:
            backups = [{}]

        data = ((b.get('backup_id', ''),
                 b.get('backup_uuid', ''),
                 b.get('backup_metadata', {}).get('hostname', ''),
                 b.get('backup_metadata', {}).get('path_to_backup', ''),
                 datetime.datetime.fromtimestamp(
                     int(b.get('backup_metadata', {}).get(
                         'time_stamp', ''))) if b.get(
                     'backup_metadata') else '',
                 b.get('backup_metadata', {}).get('curr_backup_level', '')
                 ) for b in backups)

        return columns, data


class BackupDelete(command.Command):
    """Delete a backup from the api"""
    def get_parser(self, prog_name):
        parser = super(BackupDelete, self).get_parser(prog_name)
        parser.add_argument(dest='backup_uuid',
                            help='UUID of the backup')
        return parser

    def take_action(self, parsed_args):
        self.app.client.backups.delete(parsed_args.backup_uuid)
        logging.info('Backup {0} deleted'.format(parsed_args.backup_uuid))


class BackupCreate(command.Command):
    """Create an backup from a file"""
    def get_parser(self, prog_name):
        parser = super(BackupCreate, self).get_parser(prog_name)
        parser.add_argument('--file',
                            dest='file',
                            required=True,
                            help='Path to json file with the backup')
        return parser

    def take_action(self, parsed_args):
        backup = utils.doc_from_json_file(parsed_args.file)
        backup_id = self.app.client.backups.create(backup)
        logging.info('Backup {0} created'.format(backup_id))



# (c) Copyright 2014-2016 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import requests

from oslo_serialization import jsonutils as json

from freezerclient import exceptions
from freezerclient import utils


class JobManager(object):

    def __init__(self, client, verify=True):
        self.client = client
        self.endpoint = self.client.endpoint + '/v1/jobs/'
        self.verify = verify

    @property
    def headers(self):
        return utils.create_headers_for_request(self.client.auth_token)

    def create(self, doc, job_id=''):
        job_id = job_id or doc.get('job_id', '')
        endpoint = self.endpoint + job_id
        doc['client_id'] = doc.get('client_id', '') or self.client.client_id
        r = requests.post(endpoint,
                          data=json.dumps(doc),
                          headers=self.headers,
                          verify=self.verify)
        if r.status_code != 201:
            raise exceptions.ApiClientException(r)
        job_id = r.json()['job_id']
        return job_id

    def delete(self, job_id):
        endpoint = self.endpoint + job_id
        r = requests.delete(endpoint, headers=self.headers, verify=self.verify)
        if r.status_code != 204:
            raise exceptions.ApiClientException(r)

    def list_all(self, limit=10, offset=0, search=None):
        data = json.dumps(search) if search else None
        query = {'limit': int(limit), 'offset': int(offset)}
        r = requests.get(self.endpoint, headers=self.headers,
                         params=query, data=data, verify=self.verify)
        if r.status_code != 200:
            raise exceptions.ApiClientException(r)
        return r.json()['jobs']

    def list(self, limit=10, offset=0, search={}, client_id=None):
        client_id = client_id or self.client.client_id
        new_search = search.copy()
        new_search['match'] = search.get('match', [])
        new_search['match'].append({'client_id': client_id})
        return self.list_all(limit, offset, new_search)

    def get(self, job_id):
        endpoint = self.endpoint + job_id
        r = requests.get(endpoint, headers=self.headers, verify=self.verify)
        if r.status_code == 200:
            return r.json()
        if r.status_code == 404:
            return None
        raise exceptions.ApiClientException(r)

    def update(self, job_id, update_doc):
        endpoint = self.endpoint + job_id
        r = requests.patch(endpoint,
                           headers=self.headers,
                           data=json.dumps(update_doc),
                           verify=self.verify)
        if r.status_code != 200:
            raise exceptions.ApiClientException(r)
        return r.json()['version']

    def start_job(self, job_id):
        """
        Request to start a job

        :param job_id: the id of the job to start
        :return: the response obj:
                 {
                    result: string 'success' or 'start already requested'
                 }
        """
        # endpoint /v1/jobs/{job_id}/event
        endpoint = '{0}{1}/event'.format(self.endpoint, job_id)
        doc = {"start": None}
        r = requests.post(endpoint,
                          headers=self.headers,
                          data=json.dumps(doc),
                          verify=self.verify)
        if r.status_code != 202:
            raise exceptions.ApiClientException(r)
        return r.json()

    def stop_job(self, job_id):
        """
        Request to stop a job

        :param job_id: the id of the job to start
        :return: the response obj:
                 {
                    result: string 'success' or 'stop already requested'
                 }
        """
        # endpoint /v1/jobs/{job_id}/event
        endpoint = '{0}{1}/event'.format(self.endpoint, job_id)
        doc = {"stop": None}
        r = requests.post(endpoint,
                          headers=self.headers,
                          data=json.dumps(doc),
                          verify=self.verify)
        if r.status_code != 202:
            raise exceptions.ApiClientException(r)
        return r.json()

    def abort_job(self, job_id):
        """
        Request to abort a job

        :param job_id: the id of the job to start
        :return: the response obj:
                 {
                    result: string 'success' or 'abort already requested'
                 }
        """
        # endpoint /v1/jobs/{job_id}/event
        endpoint = '{0}{1}/event'.format(self.endpoint, job_id)
        doc = {"abort": None}
        r = requests.post(endpoint,
                          headers=self.headers,
                          data=json.dumps(doc),
                          verify=self.verify)
        if r.status_code != 202:
            raise exceptions.ApiClientException(r)
        return r.json()



# (c) Copyright 2018 ZTE Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from unittest import mock

from freezerclient import exceptions
from freezerclient.v2.managers import clients


class TestClientManager(unittest.TestCase):
    def setUp(self):
        self.mock_client = mock.Mock()
        self.mock_client.endpoint = 'http://testendpoint:9999'
        self.mock_client.project_id = 'tecs'
        self.mock_client.auth_token = 'testtoken'
        self.r = clients.ClientManager(self.mock_client)

    @mock.patch('freezerclient.v2.managers.clients.requests')
    def test_create(self, mock_requests):
        self.assertEqual('http://testendpoint:9999/v2/tecs/clients/',
                         self.r.endpoint)
        self.assertEqual({'X-Auth-Token': 'testtoken',
                          'Content-Type': 'application/json',
                          'Accept': 'application/json'},
                         self.r.headers)

    @mock.patch('freezerclient.v2.managers.clients.requests')
    def test_create_ok(self, mock_requests):
        mock_response = mock.Mock()
        mock_response.status_code = 201
        mock_response.json.return_value = {'client_id': 'qwerqwer'}
        mock_requests.post.return_value = mock_response
        retval = self.r.create(client_info={'client': 'metadata'})
        self.assertEqual('qwerqwer', retval)

    @mock.patch('freezerclient.v2.managers.clients.requests')
    def test_create_fail_when_api_return_error_code(self, mock_requests):
        mock_response = mock.Mock()
        mock_response.status_code = 500
        mock_requests.post.return_value = mock_response
        self.assertRaises(exceptions.ApiClientException, self.r.create,
                          {'client': 'metadata'})

    @mock.patch('freezerclient.v2.managers.clients.requests')
    def test_delete_ok(self, mock_requests):
        mock_response = mock.Mock()
        mock_response.status_code = 204
        mock_requests.delete.return_value = mock_response
        retval = self.r.delete('test_client_id')
        self.assertIsNone(retval)

    @mock.patch('freezerclient.v2.managers.clients.requests')
    def test_delete_fail(self, mock_requests):
        mock_response = mock.Mock()
        mock_response.status_code = 500
        mock_requests.delete.return_value = mock_response
        self.assertRaises(exceptions.ApiClientException, self.r.delete,
                          'test_client_id')

    @mock.patch('freezerclient.v2.managers.clients.requests')
    def test_get_ok(self, mock_requests):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {'client_id': 'qwerqwer'}
        mock_requests.get.return_value = mock_response
        retval = self.r.get('test_client_id')
        self.assertEqual({'client_id': 'qwerqwer'}, retval)

    @mock.patch('freezerclient.v2.managers.clients.requests')
    def test_get_none(self, mock_requests):
        mock_response = mock.Mock()
        mock_response.status_code = 404
        mock_requests.get.return_value = mock_response
        retval = self.r.get('test_client_id')
        self.assertIsNone(retval)

    @mock.patch('freezerclient.v2.managers.clients.requests')
    def test_get_raises_ApiClientException_on_error_not_404(self,
                                                            mock_requests):
        mock_response = mock.Mock()
        mock_response.status_code = 500
        mock_requests.get.return_value = mock_response
        self.assertRaises(exceptions.ApiClientException, self.r.get,
                          'test_client_id')

    @mock.patch('freezerclient.v2.managers.clients.requests')
    def test_list_ok(self, mock_requests):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        client_list = [{'client_id_0': 'qwerqwer'},
                       {'client_id_1': 'asdfasdf'}]
        mock_response.json.return_value = {'clients': client_list}
        mock_requests.get.return_value = mock_response
        retval = self.r.list()
        self.assertEqual(client_list, retval)

    @mock.patch('freezerclient.v2.managers.clients.requests')
    def test_list_error(self, mock_requests):
        mock_response = mock.Mock()
        mock_response.status_code = 404
        client_list = [{'client_id_0': 'qwerqwer'},
                       {'client_id_1': 'asdfasdf'}]
        mock_response.json.return_value = {'clients': client_list}
        mock_requests.get.return_value = mock_response
        self.assertRaises(exceptions.ApiClientException, self.r.list)



# (c) Copyright 2014-2016 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from cliff import command
from cliff import lister
from cliff import show

from freezerclient import exceptions
from freezerclient import utils


logging = logging.getLogger(__name__)


class ActionShow(show.ShowOne):
    """Show a single action """
    def get_parser(self, prog_name):
        parser = super(ActionShow, self).get_parser(prog_name)
        parser.add_argument(dest='action_id',
                            help='ID of the action')
        return parser

    def take_action(self, parsed_args):
        action = self.app.client.actions.get(parsed_args.action_id)

        if not action:
            raise exceptions.ApiClientException('Action not found')

        column = (
            'Action ID',
            'Name',
            'Action',
            'Mode',
            'Path to Backup or Restore',
            'Storage',
            'Snapshot',
            'Container',
            'Log_file',
            'Remove_older_than',
            'Max_retries_interval',
            'Max_retries',
            'User_id',
            'Project_id'
        )

        data = (
            action.get('action_id'),
            action.get('freezer_action', {}).get('backup_name', ''),
            action.get('freezer_action', {}).get('action', 'backup'),
            action.get('freezer_action', {}).get('mode', 'fs'),
            action.get('freezer_action', {}).get('path_to_backup', ''),
            action.get('freezer_action', {}).get('storage', 'swift'),
            action.get('freezer_action', {}).get('snapshot', 'False'),
            action.get('freezer_action', {}).get('container', ''),
            action.get('freezer_action', {}).get('log_file', ''),
            action.get('freezer_action', {}).get('remove_older_than', '365'),
            action.get('max_retries_interval', '6'),
            action.get('max_retries', '5'),
            action.get('user_id', ''),
            action.get('project_id', ''),
        )

        return column, data


class ActionList(lister.Lister):
    """List all actions for your user"""
    def get_parser(self, prog_name):
        parser = super(ActionList, self).get_parser(prog_name)

        parser.add_argument(
            '--limit',
            dest='limit',
            default=100,
            help='Specify a limit for search query',
        )

        parser.add_argument(
            '--offset',
            dest='offset',
            default=0,
            help='',
        )

        parser.add_argument(
            '--search',
            dest='search',
            default='',
            help='Define a filter for the query',
        )
        return parser

    def take_action(self, parsed_args):
        search = utils.prepare_search(parsed_args.search)

        actions = self.app.client.actions.list(
            limit=parsed_args.limit,
            offset=parsed_args.offset,
            search=search
        )

        columns = ('Action ID', 'Name', 'Action',
                   'Path to Backup or Restore', 'Mode', 'Storage', 'snapshot')

        # Print empty table if no actions found
        if not actions:
            actions = [{}]
            data = ((action.get('action-id', ''),
                     action.get('freezer_action', {}).get('backup_name', ''),
                     action.get('freezer_action', {}).get('action', ''),
                     action.get('freezer_action', {}).get(
                         'path_to_backup', ''),
                     action.get('freezer_action', {}).get('mode', ''),
                     action.get('freezer_action', {}).get('storage', ''),
                     action.get('freezer_action', {}).get('snapshot', '')
                     ) for action in actions)
        else:
            data = ((action.get('action_id'),
                     action.get('freezer_action', {}).get('backup_name', ''),
                     action.get('freezer_action', {}).get('action', 'backup'),
                     action.get('freezer_action', {}).get(
                         'path_to_backup', ''),
                     action.get('freezer_action', {}).get('mode', 'fs'),
                     action.get('freezer_action', {}).get('storage', 'swift'),
                     action.get('freezer_action', {}).get('snapshot', 'False')
                     ) for action in actions)

        return columns, data


class ActionDelete(command.Command):
    """Delete an action from the api"""
    def get_parser(self, prog_name):
        parser = super(ActionDelete, self).get_parser(prog_name)
        parser.add_argument(dest='action_id',
                            help='ID of the action')
        return parser

    def take_action(self, parsed_args):
        self.app.client.actions.delete(parsed_args.action_id)
        logging.info('Action {0} deleted'.format(parsed_args.action_id))


class ActionCreate(command.Command):
    """Create an action from a file"""
    def get_parser(self, prog_name):
        parser = super(ActionCreate, self).get_parser(prog_name)
        parser.add_argument('--file',
                            dest='file',
                            required=True,
                            help='Path to json file with the action')
        return parser

    def take_action(self, parsed_args):
        action = utils.doc_from_json_file(parsed_args.file)
        action_id = self.app.client.actions.create(action)
        logging.info('Action {0} created'.format(action_id))


class ActionUpdate(command.Command):
    """Update an action from a file"""
    def get_parser(self, prog_name):
        parser = super(ActionUpdate, self).get_parser(prog_name)
        parser.add_argument(dest='action_id',
                            help='ID of the session')

        parser.add_argument(dest='file',
                            help='Path to json file with the action')
        return parser

    def take_action(self, parsed_args):
        action = utils.doc_from_json_file(parsed_args.file)
        self.app.client.actions.update(parsed_args.action_id, action)
        logging.info('Action {0} updated'.format(parsed_args.action_id))



# (c) Copyright 2014-2016 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import os
import sys

from cliff import app
from cliff import commandmanager

import freezerclient
from freezerclient import utils

log = logging.getLogger(__name__)
# Suppress output from requests
logging.getLogger('requests').setLevel(logging.WARN)


class FreezerCommandManager(commandmanager.CommandManager):
    """All commands available for the shell are registered here"""
    if utils.check_api_version() == '1':
        from freezerclient.v1 import actions
        from freezerclient.v1 import backups
        from freezerclient.v1 import clients
        from freezerclient.v1 import jobs
        from freezerclient.v1 import sessions
    else:
        from freezerclient.v2 import actions
        from freezerclient.v2 import backups
        from freezerclient.v2 import clients
        from freezerclient.v2 import jobs
        from freezerclient.v2 import sessions

    SHELL_COMMANDS = {
        'job-show': jobs.JobShow,
        'job-list': jobs.JobList,
        'job-create': jobs.JobCreate,
        'job-get': jobs.JobGet,
        'job-delete': jobs.JobDelete,
        'job-start': jobs.JobStart,
        'job-stop': jobs.JobStop,
        'job-abort': jobs.JobAbort,
        'job-update': jobs.JobUpdate,
        'client-list': clients.ClientList,
        'client-show': clients.ClientShow,
        'client-register': clients.ClientRegister,
        'client-delete': clients.ClientDelete,
        'backup-list': backups.BackupList,
        'backup-show': backups.BackupShow,
        'backup-delete': backups.BackupDelete,
        'backup-create': backups.BackupCreate,
        'session-list': sessions.SessionList,
        'session-show': sessions.SessionShow,
        'session-create': sessions.SessionCreate,
        'session-delete': sessions.SessionDelete,
        'session-add-job': sessions.SessionAddJob,
        'session-remove-job': sessions.SessionRemoveJob,
        'session-update': sessions.SessionUpdate,
        'session-start': sessions.SessionStart,
        'action-show': actions.ActionShow,
        'action-list': actions.ActionList,
        'action-delete': actions.ActionDelete,
        'action-create': actions.ActionCreate,
        'action-update': actions.ActionUpdate
    }

    def load_commands(self, namespace):
        for name, command_class in self.SHELL_COMMANDS.items():
            self.add_command(name, command_class)


class FreezerShell(app.App):
    def __init__(self):
        super(FreezerShell, self).__init__(
            description='Python Freezer Client',
            version=freezerclient.__version__,
            deferred_help=True,
            command_manager=FreezerCommandManager('freezerclient'),
        )

    def build_option_parser(self, description, version):
        parser = super(FreezerShell, self).build_option_parser(description,
                                                               version)
        parser.add_argument(
            '--os-auth-url',
            dest='os_auth_url',
            default=os.environ.get('OS_AUTH_URL'),
            help='Specify identity endpoint',
        )

        parser.add_argument(
            '--os-backup-api-version',
            dest='os_backup_api_version',
            default=os.environ.get('OS_BACKUP_API_VERSION', '2'),
            help='Backup API version: 1 or 2'
        )

        parser.add_argument(
            '--os-backup-url',
            dest='os_backup_url',
            default=os.environ.get('OS_BACKUP_URL'),
            help='Specify the Freezer backup service endpoint to use'
        )

        parser.add_argument(
            '--os-endpoint-type',
            dest='os_endpoint_type',
            default=os.environ.get('OS_ENDPOINT_TYPE') or 'public',
            help='''Endpoint type to select. Valid endpoint types:
                    "public" or "publicURL", "internal" or "internalURL",
                    "admin" or "adminURL"'''
        )

        parser.add_argument(
            '--os-password',
            dest='os_password',
            default=os.environ.get('OS_PASSWORD'),
            help='''Password used for authentication with the OpenStack
                    Identity service'''
        )

        parser.add_argument(
            '--os-username',
            dest='os_username',
            default=os.environ.get('OS_USERNAME'),
            help='''Name used for authentication with the OpenStack
                    Identity service'''
        )

        parser.add_argument(
            '--os-token',
            dest='os_token',
            default=os.environ.get('OS_TOKEN'),
            help='''Specify an existing token to use instead of retrieving
                    one via authentication'''
        )

        parser.add_argument(
            '--os-project-domain-name',
            dest='os_project_domain_name',
            default=os.environ.get('OS_PROJECT_DOMAIN_NAME'),
            help='Domain name containing project'
        )

        parser.add_argument(
            '--os-project-domain-id',
            dest='os_project_domain_id',
            default=os.environ.get('OS_PROJECT_DOMAIN_ID'),
            help='OpenStack project domain ID. '
                 'Defaults to env[OS_PROJECT_ID].'
        )

        parser.add_argument(
            '--os-project-name',
            dest='os_project_name',
            default=os.environ.get('OS_PROJECT_NAME'),
            help='Project name to scope to'
        )

        parser.add_argument(
            '--os-region-name',
            dest='os_region_name',
            default=os.environ.get('OS_REGION_NAME'),
            help='Specify the region to use'
        )

        parser.add_argument(
            '--os-project-id',
            dest='os_project_id',
            default=os.environ.get('OS_PROJECT_ID'),
            help='Project to request authorization on'
        )

        parser.add_argument(
            '--os-user-domain-name',
            dest='os_user_domain_name',
            default=os.environ.get('OS_USER_DOMAIN_NAME'),
            help='User domain name'
        )

        parser.add_argument(
            '--os-user-domain-id',
            dest='os_user_domain_id',
            default=os.environ.get('OS_USER_DOMAIN_ID'),
            help='OpenStack user domain ID. '
                 'Defaults to env[OS_USER_DOMAIN_ID].'
        )

        parser.add_argument(
            '-k', '--insecure',
            dest='insecure',
            action='store_true',
            default=os.environ.get('OS_INSECURE'),
            help='use python-freezerclient with insecure connections'
        )

        parser.add_argument(
            '--os-cacert',
            dest='os_cacert',
            default=os.environ.get('OS_CACERT'),
            help='''Path of CA TLS certificate(s) used to verify the
                    remote server's certificate. Without this option
                    freezer looks for the default system CA certificates.'''
        )

        parser.add_argument(
            '--os-cert',
            dest='os_cert',
            default=os.environ.get('OS_CERT'),
            help='''Path of CERT TLS certificate(s) used to verify the
                    remote server's certificate.1'''
        )

        return parser

    @property
    def client(self):
        """Factory function to create a new freezer service client.

        The returned client will be either a V1 or V2 client.

        :return: freezerclient object
        """
        opts = {
            'token': self.options.os_token,
            'username': self.options.os_username,
            'password': self.options.os_password,
            'tenant_name': self.options.os_project_name,
            'auth_url': self.options.os_auth_url,
            'endpoint': self.options.os_backup_url,
            'endpoint_type': self.options.os_endpoint_type,
            'project_name': self.options.os_project_name,
            'project_id': self.options.os_project_id,
            'user_domain_name': self.options.os_user_domain_name,
            'user_domain_id': self.options.os_user_domain_id,
            'project_domain_name': self.options.os_project_domain_name,
            'project_domain_id': self.options.os_project_domain_id,
            'cert': self.options.os_cert,
            'cacert': self.options.os_cacert,
            'insecure': self.options.insecure
        }
        return utils.get_client_instance(
            opts, api_version=self.options.os_backup_api_version)


def main(argv=sys.argv[1:]):
    return FreezerShell().run(argv)


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))



# (c) Copyright 2018 ZTE Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from unittest import mock

from freezerclient import client
from freezerclient import v1
from freezerclient import v2


class ClientTest(unittest.TestCase):
    def setUp(self):
        self.session = mock.Mock()

    def test_client_v1_endpoint(self):
        gc = client.Client(version='1', endpoint='http://freezer.org',
                           session=self.session)
        self.assertEqual("http://freezer.org", gc.opts.os_backup_url)
        self.assertIsInstance(gc, v1.client.Client)

    def test_client_v1_auth_url(self):
        gc = client.Client(version='1', auth_url='http://example.com',
                           endpoint='http://freezer.org',
                           session=self.session)
        self.assertEqual("http://freezer.org", gc.opts.os_backup_url)
        self.assertIsInstance(gc, v1.client.Client)

    def test_client_v1_username(self):
        gc = client.Client('1', endpoint='http://freezer.org',
                           username='caihui', session=self.session)
        self.assertEqual("caihui", gc.opts.os_username)
        self.assertIsInstance(gc, v1.client.Client)

    def test_client_v1_password(self):
        gc = client.Client('1', password='password',
                           endpoint='http://freezer.org',
                           session=self.session)
        self.assertEqual("password", gc.opts.os_password)
        self.assertIsInstance(gc, v1.client.Client)

    def test_client_v2_auth_url(self):
        gc = client.Client(version='2', auth_url='http://example.com',
                           session=self.session)
        self.assertEqual("http://example.com", gc.opts.os_auth_url)
        self.assertIsInstance(gc, v2.client.Client)

    def test_client_v2_endpoint(self):
        gc = client.Client(version='2', endpoint='http://freezer.org',
                           session=self.session)
        self.assertEqual("http://freezer.org", gc.opts.os_backup_url)
        self.assertIsInstance(gc, v2.client.Client)

    def test_client_v2_username(self):
        gc = client.Client('2', username='caihui', session=self.session)
        self.assertEqual("caihui", gc.opts.os_username)
        self.assertIsInstance(gc, v2.client.Client)

    def test_client_v2_password(self):
        gc = client.Client('2', password='password', session=self.session)
        self.assertEqual("password", gc.opts.os_password)
        self.assertIsInstance(gc, v2.client.Client)

    def test_client_v2_porject_name(self):
        gc = client.Client('2', project_name='tecs', session=self.session)
        self.assertEqual("tecs", gc.opts.os_project_name)
        self.assertIsInstance(gc, v2.client.Client)

    def test_client_default_version(self):
        gc = client.Client(session=self.session)
        self.assertIsInstance(gc, v2.client.Client)



# (c) Copyright 2018 ZTE Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from unittest import mock

from oslo_serialization import jsonutils as json

from freezerclient import exceptions
from freezerclient.v2.managers import jobs


class TestJobManager(unittest.TestCase):
    def setUp(self):
        self.mock_client = mock.Mock()
        self.mock_response = mock.Mock()
        self.mock_client.endpoint = 'http://testendpoint:9999'
        self.mock_client.project_id = 'tecs'
        self.mock_client.auth_token = 'testtoken'
        self.mock_client.client_id = 'test_client_id_78900987'
        self.job_manager = jobs.JobManager(self.mock_client)
        self.headers = {
            'X-Auth-Token': 'testtoken',
            'Content-Type': 'application/json',
            'Accept': 'application/json'
        }

    @mock.patch('freezerclient.v2.managers.jobs.requests')
    def test_create(self, mock_requests):
        self.assertEqual('http://testendpoint:9999/v2/tecs/jobs/',
                         self.job_manager.endpoint)
        self.assertEqual({'X-Auth-Token': 'testtoken',
                          'Content-Type': 'application/json',
                          'Accept': 'application/json'},
                         self.job_manager.headers)

    @mock.patch('freezerclient.v2.managers.jobs.requests')
    def test_create_ok(self, mock_requests):
        self.mock_response.status_code = 201
        self.mock_response.json.return_value = {'job_id': 'qwerqwer'}
        mock_requests.post.return_value = self.mock_response
        retval = self.job_manager.create({'job': 'metadata'})
        self.assertEqual('qwerqwer', retval)

    @mock.patch('freezerclient.v2.managers.jobs.json')
    @mock.patch('freezerclient.v2.managers.jobs.requests')
    def test_create_adds_client_id_if_not_provided(self, mock_requests,
                                                   mock_json):
        self.mock_response.status_code = 201
        self.mock_response.json.return_value = {'job_id': 'qwerqwer'}
        mock_json.dumps.return_value = {'job': 'mocked'}
        mock_requests.post.return_value = self.mock_response

        retval = self.job_manager.create({'job': 'metadata'})

        mock_json.dumps.assert_called_with({
            'job': 'metadata', 'client_id': 'test_client_id_78900987'
        })
        self.assertEqual('qwerqwer', retval)

    @mock.patch('freezerclient.v2.managers.jobs.json')
    @mock.patch('freezerclient.v2.managers.jobs.requests')
    def test_create_leaves_provided_client_id(self, mock_requests, mock_json):
        self.mock_response.status_code = 201
        self.mock_response.json.return_value = {'job_id': 'qwerqwer'}
        mock_json.dumps.return_value = {'job': 'mocked'}
        mock_requests.post.return_value = self.mock_response

        retval = self.job_manager.create(
            {'job': 'metadata', 'client_id': 'parmenide'})

        mock_json.dumps.assert_called_with({'job': 'metadata',
                                            'client_id': 'parmenide'})
        self.assertEqual('qwerqwer', retval)

    @mock.patch('freezerclient.v2.managers.jobs.requests')
    def test_create_fail_when_api_return_error_code(self, mock_requests):
        self.mock_response.status_code = 500
        mock_requests.post.return_value = self.mock_response
        self.assertRaises(exceptions.ApiClientException,
                          self.job_manager.create, {'job': 'metadata'})

    @mock.patch('freezerclient.v2.managers.jobs.requests')
    def test_delete_ok(self, mock_requests):
        self.mock_response.status_code = 204
        mock_requests.delete.return_value = self.mock_response
        retval = self.job_manager.delete('test_job_id')
        self.assertIsNone(retval)

    @mock.patch('freezerclient.v2.managers.jobs.requests')
    def test_delete_fail(self, mock_requests):
        self.mock_response.status_code = 500
        mock_requests.delete.return_value = self.mock_response
        self.assertRaises(exceptions.ApiClientException,
                          self.job_manager.delete, 'test_job_id')

    @mock.patch('freezerclient.v2.managers.jobs.requests')
    def test_get_ok(self, mock_requests):
        self.mock_response.status_code = 200
        self.mock_response.json.return_value = {'job_id': 'qwerqwer'}
        mock_requests.get.return_value = self.mock_response
        retval = self.job_manager.get('test_job_id')
        self.assertEqual({'job_id': 'qwerqwer'}, retval)

    @mock.patch('freezerclient.v2.managers.jobs.requests')
    def test_get_fails_on_error_different_from_404(self, mock_requests):
        self.mock_response.status_code = 500
        mock_requests.get.return_value = self.mock_response
        self.assertRaises(exceptions.ApiClientException, self.job_manager.get,
                          'test_job_id')

    @mock.patch('freezerclient.v2.managers.jobs.requests')
    def test_get_none(self, mock_requests):
        self.mock_response.status_code = 404
        mock_requests.get.return_value = self.mock_response
        retval = self.job_manager.get('test_job_id')
        self.assertIsNone(retval)

    @mock.patch('freezerclient.v2.managers.jobs.requests')
    def test_list_ok(self, mock_requests):
        self.mock_response.status_code = 200
        job_list = [{'job_id_0': 'bomboloid'}, {'job_id_1': 'asdfasdf'}]
        self.mock_response.json.return_value = {'jobs': job_list}
        mock_requests.get.return_value = self.mock_response
        retval = self.job_manager.list()
        self.assertEqual(job_list, retval)

    @mock.patch('freezerclient.v2.managers.jobs.requests')
    def test_list_error(self, mock_requests):
        self.mock_response.status_code = 404
        job_list = [{'job_id_0': 'bomboloid'}, {'job_id_1': 'asdfasdf'}]
        self.mock_response.json.return_value = {'clients': job_list}
        mock_requests.get.return_value = self.mock_response
        self.assertRaises(exceptions.ApiClientException, self.job_manager.list)

    @mock.patch('freezerclient.v2.managers.jobs.requests')
    def test_update_ok(self, mock_requests):
        self.mock_response.status_code = 200
        self.mock_response.json.return_value = {
            "patch": {"status": "bamboozled"},
            "version": 12,
            "job_id": "d454beec-1f3c-4d11-aa1a-404116a40502"
        }
        mock_requests.patch.return_value = self.mock_response
        retval = self.job_manager.update(
            'd454beec-1f3c-4d11-aa1a-404116a40502', {'status': 'bamboozled'})
        self.assertEqual(12, retval)

    @mock.patch('freezerclient.v2.managers.jobs.requests')
    def test_update_raise_MetadataUpdateFailure_when_api_return_error_code(
            self, mock_requests):
        self.mock_response.json.return_value = {
            "patch": {"status": "bamboozled"},
            "version": 12,
            "job_id": "d454beec-1f3c-4d11-aa1a-404116a40502"
        }
        self.mock_response.status_code = 404
        self.mock_response.text = (
            '{"title": "Not Found","description":"No document found with ID '
            'd454beec-1f3c-4d11-aa1a-404116a40502x"}'
        )
        mock_requests.patch.return_value = self.mock_response
        self.assertRaises(exceptions.ApiClientException,
                          self.job_manager.update,
                          'd454beec-1f3c-4d11-aa1a-404116a40502',
                          {'status': 'bamboozled'})

    @mock.patch('freezerclient.v2.managers.jobs.requests')
    def test_start_job_posts_proper_data(self, mock_requests):
        job_id = 'jobdfsfnqwerty1234'
        self.mock_response.status_code = 202
        self.mock_response.json.return_value = {'result': 'success'}
        mock_requests.post.return_value = self.mock_response
        # /v2/{project_id}/jobs/{job_id}/event

        endpoint = '{0}/v2/{1}/jobs/{2}/event'.format(
            self.mock_client.endpoint,
            self.mock_client.project_id,
            job_id)
        data = {"start": None}
        retval = self.job_manager.start_job(job_id)
        self.assertEqual({'result': 'success'}, retval)

        args = mock_requests.post.call_args[0]
        kwargs = mock_requests.post.call_args[1]
        self.assertEqual(endpoint, args[0])
        self.assertEqual(data, json.loads(kwargs['data']))
        self.assertEqual(self.headers, kwargs['headers'])

    @mock.patch('freezerclient.v2.managers.jobs.requests')
    def test_start_job_raise_ApiClientException_when_api_return_error_code(
            self, mock_requests):
        job_id = 'jobdfsfnqwerty1234'
        self.mock_response.status_code = 500
        self.mock_response.json.return_value = {'result': 'success'}
        mock_requests.post.return_value = self.mock_response
        self.assertRaises(exceptions.ApiClientException,
                          self.job_manager.start_job, job_id)

    @mock.patch('freezerclient.v2.managers.jobs.requests')
    def test_stop_job_posts_proper_data(self, mock_requests):
        job_id = 'jobdfsfnqwerty1234'
        self.mock_response.status_code = 202
        self.mock_response.json.return_value = {'result': 'success'}
        mock_requests.post.return_value = self.mock_response
        # /v2/{project_id}/jobs/{job_id}/event

        endpoint = '{0}/v2/{1}/jobs/{2}/event'.format(
            self.mock_client.endpoint,
            self.mock_client.project_id,
            job_id)
        data = {"stop": None}
        retval = self.job_manager.stop_job(job_id)
        self.assertEqual({'result': 'success'}, retval)

        args = mock_requests.post.call_args[0]
        kwargs = mock_requests.post.call_args[1]
        self.assertEqual(endpoint, args[0])
        self.assertEqual(data, json.loads(kwargs['data']))
        self.assertEqual(self.headers, kwargs['headers'])

    @mock.patch('freezerclient.v2.managers.jobs.requests')
    def test_stop_job_raise_ApiClientException_when_api_return_error_code(
            self, mock_requests):
        job_id = 'jobdfsfnqwerty1234'
        self.mock_response.status_code = 500
        self.mock_response.json.return_value = {'result': 'success'}
        mock_requests.post.return_value = self.mock_response
        self.assertRaises(exceptions.ApiClientException,
                          self.job_manager.start_job, job_id)

    @mock.patch('freezerclient.v2.managers.jobs.requests')
    def test_abort_job_posts_proper_data(self, mock_requests):
        job_id = 'jobdfsfnqwerty1234'
        self.mock_response.status_code = 202
        self.mock_response.json.return_value = {'result': 'success'}
        mock_requests.post.return_value = self.mock_response
        # /v2/{project_id}/jobs/{job_id}/event

        endpoint = '{0}/v2/{1}/jobs/{2}/event'.format(
            self.mock_client.endpoint,
            self.mock_client.project_id,
            job_id)
        data = {"abort": None}
        retval = self.job_manager.abort_job(job_id)
        self.assertEqual({'result': 'success'}, retval)

        args = mock_requests.post.call_args[0]
        kwargs = mock_requests.post.call_args[1]
        self.assertEqual(endpoint, args[0])
        self.assertEqual(data, json.loads(kwargs['data']))
        self.assertEqual(self.headers, kwargs['headers'])

    @mock.patch('freezerclient.v2.managers.jobs.requests')
    def test_abort_job_raise_ApiClientException_when_api_return_error_code(
            self, mock_requests):
        job_id = 'jobdfsfnqwerty1234'
        self.mock_response.status_code = 500
        self.mock_response.json.return_value = {'result': 'success'}
        mock_requests.post.return_value = self.mock_response
        self.assertRaises(exceptions.ApiClientException,
                          self.job_manager.abort_job, job_id)



# Copyright (c) 2018 ZTE Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from unittest import mock

from keystoneauth1 import loading as kaloading

from freezerclient.v2 import client


class TestClientMock(unittest.TestCase):

    @mock.patch.object(kaloading.session, 'Session', autospec=True)
    @mock.patch.object(kaloading, 'get_plugin_loader', autospec=True)
    def test_client_new(self, mock_ks_loader, mock_ks_session):
        session = mock_ks_session.return_value.load_from_options.return_value
        mock_ks_loader.return_value.load_from_options.return_value = 'auth'
        c = client.Client(endpoint='blabla', auth_url='blabla')
        self.assertIsInstance(c, client.Client)
        self.assertEqual(session, c.session)

    @mock.patch.object(kaloading.session, 'Session', autospec=True)
    @mock.patch.object(kaloading, 'get_plugin_loader', autospec=True)
    def test_client_new_with_kwargs_session(self, mock_ks_loader,
                                            mock_ks_session):
        mock_ks_loader.return_value.load_from_options.return_value = 'auth'
        mock_session = mock.Mock()
        kwargs = {'token': 'alpha',
                  'username': 'bravo',
                  'password': 'charlie',
                  'tenant_name': 'delta',
                  'auth_url': 'echo',
                  'endpoint': 'golf',
                  'session': mock_session}
        c = client.Client(**kwargs)
        self.assertIsInstance(c, client.Client)
        self.assertEqual('alpha', c.opts.os_token)
        self.assertEqual('bravo', c.opts.os_username)
        self.assertEqual('charlie', c.opts.os_password)
        self.assertEqual('delta', c.opts.os_tenant_name)
        self.assertEqual('echo', c.opts.os_auth_url)
        self.assertEqual(mock_session, c._session)
        self.assertEqual(mock_session, c.session)
        self.assertEqual('golf', c.endpoint)

    @mock.patch.object(kaloading.session, 'Session', autospec=True)
    @mock.patch.object(kaloading, 'get_plugin_loader', autospec=True)
    def test_client_new_with_kwargs_usename_password(self, mock_ks_loader,
                                                     mock_ks_session):
        session = mock_ks_session.return_value.load_from_options.return_value
        mock_ks_loader.return_value.load_from_options.return_value = 'auth'
        kwargs = {'auth_url': 'one',
                  'project_id': 'two',
                  'tenant_name': 'three',
                  'project_name': 'four',
                  'user_domain_id': 'five',
                  'user_domain_name': 'six',
                  'project_domain_id': 'senven',
                  'project_domain_name': 'eight',
                  'username': 'nine',
                  'password': 'ten'}

        c = client.Client(**kwargs)
        self.assertIsInstance(c, client.Client)
        self.assertEqual('one', c.opts.os_auth_url)
        self.assertEqual('two', c.opts.os_project_id)
        self.assertEqual('three', c.opts.os_tenant_name)
        self.assertEqual('four', c.opts.os_project_name)
        self.assertEqual('five', c.opts.os_user_domain_id)
        self.assertEqual('six', c.opts.os_user_domain_name)
        self.assertEqual('senven', c.opts.os_project_domain_id)
        self.assertEqual('eight', c.opts.os_project_domain_name)
        self.assertEqual('nine', c.opts.os_username)
        self.assertEqual('ten', c.opts.os_password)
        self.assertEqual(session, c.session)

    @mock.patch.object(kaloading.session, 'Session', autospec=True)
    @mock.patch.object(kaloading, 'get_plugin_loader', autospec=True)
    def test_client_new_with_kwargs_token(self, mock_ks_loader,
                                          mock_ks_session):
        session = mock_ks_session.return_value.load_from_options.return_value
        mock_ks_loader.return_value.load_from_options.return_value = 'auth'
        kwargs = {'auth_url': 'one',
                  'project_id': 'two',
                  'tenant_name': 'three',
                  'project_name': 'four',
                  'user_domain_id': 'five',
                  'user_domain_name': 'six',
                  'project_domain_id': 'senven',
                  'project_domain_name': 'eight',
                  'token': 'nine'}

        c = client.Client(**kwargs)
        self.assertIsInstance(c, client.Client)
        self.assertEqual('one', c.opts.os_auth_url)
        self.assertEqual('two', c.opts.os_project_id)
        self.assertEqual('three', c.opts.os_tenant_name)
        self.assertEqual('four', c.opts.os_project_name)
        self.assertEqual('five', c.opts.os_user_domain_id)
        self.assertEqual('six', c.opts.os_user_domain_name)
        self.assertEqual('senven', c.opts.os_project_domain_id)
        self.assertEqual('eight', c.opts.os_project_domain_name)
        self.assertEqual('nine', c.opts.os_token)
        self.assertEqual(session, c.session)

    @mock.patch.object(kaloading.session, 'Session', autospec=True)
    @mock.patch.object(kaloading, 'get_plugin_loader', autospec=True)
    def test_get_token(self, mock_ks_loader, mock_ks_session):
        mock_ks_loader.return_value.load_from_options.return_value = 'auth'
        mock_session = mock.Mock()
        mock_session.get_token.return_value = 'antaniX2'
        c = client.Client(session=mock_session, endpoint='justtest',
                          auth_url='blabla')
        self.assertIsInstance(c, client.Client)
        self.assertEqual(c.auth_token, 'antaniX2')

    @mock.patch('freezerclient.v2.client.socket')
    @mock.patch.object(kaloading.session, 'Session', autospec=True)
    @mock.patch.object(kaloading, 'get_plugin_loader', autospec=True)
    def test_get_client_id(self, mock_ks_loader, mock_ks_session,
                           mock_socket):
        mock_ks_loader.return_value.load_from_options.return_value = 'auth'
        mock_socket.gethostname.return_value = 'parmenide'
        mock_session = mock.Mock()
        mock_session.get_project_id.return_value = 'H2O'
        c = client.Client(session=mock_session, endpoint='justtest',
                          auth_url='blabla')
        self.assertIsInstance(c, client.Client)
        self.assertEqual(c.client_id, 'H2O_parmenide')



# (c) Copyright 2018 ZTE Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from unittest import mock

from oslo_serialization import jsonutils as json

from freezerclient import exceptions
from freezerclient.v2.managers import sessions


class TestSessionManager(unittest.TestCase):
    def setUp(self):
        self.mock_client = mock.Mock()
        self.mock_response = mock.Mock()
        self.mock_client.endpoint = 'http://testendpoint:9999'
        self.mock_client.project_id = 'tecs'
        self.mock_client.auth_token = 'testtoken'
        self.mock_client.client_id = 'test_client_id_78900987'
        self.session_manager = sessions.SessionManager(self.mock_client)
        self.endpoint = 'http://testendpoint:9999/v2/tecs/sessions/'
        self.headers = {
            'X-Auth-Token': 'testtoken',
            'Content-Type': 'application/json',
            'Accept': 'application/json'
        }

    @mock.patch('freezerclient.v2.managers.sessions.requests')
    def test_create(self, mock_requests):
        self.assertEqual(self.endpoint, self.session_manager.endpoint)
        self.assertEqual(self.headers, self.session_manager.headers)

    @mock.patch('freezerclient.v2.managers.sessions.requests')
    def test_create_ok(self, mock_requests):
        self.mock_response.status_code = 201
        self.mock_response.json.return_value = {'session_id': 'qwerqwer'}
        mock_requests.post.return_value = self.mock_response
        retval = self.session_manager.create({'session': 'metadata'})
        self.assertEqual('qwerqwer', retval)

    @mock.patch('freezerclient.v2.managers.sessions.requests')
    def test_create_raise_ApiClientException_when_api_return_error_code(
            self, mock_requests):
        self.mock_response.status_code = 500
        mock_requests.post.return_value = self.mock_response
        self.assertRaises(exceptions.ApiClientException,
                          self.session_manager.create, {'session': 'metadata'})

    @mock.patch('freezerclient.v2.managers.sessions.requests')
    def test_delete_ok(self, mock_requests):
        self.mock_response.status_code = 204
        mock_requests.delete.return_value = self.mock_response
        retval = self.session_manager.delete('test_session_id')
        self.assertIsNone(retval)

    @mock.patch('freezerclient.v2.managers.sessions.requests')
    def test_delete_raise_ApiClientException_when_api_return_error_code(
            self, mock_requests):
        self.mock_response.status_code = 500
        mock_requests.delete.return_value = self.mock_response
        self.assertRaises(exceptions.ApiClientException,
                          self.session_manager.delete, 'test_session_id')

    @mock.patch('freezerclient.v2.managers.sessions.requests')
    def test_get_ok(self, mock_requests):
        self.mock_response.status_code = 200
        self.mock_response.json.return_value = {'session_id': 'qwerqwer'}
        mock_requests.get.return_value = self.mock_response
        retval = self.session_manager.get('test_session_id')
        self.assertEqual({'session_id': 'qwerqwer'}, retval)

    @mock.patch('freezerclient.v2.managers.sessions.requests')
    def test_get_raise_ApiClientException_when_api_return_error_diff_from_404(
            self, mock_requests):
        self.mock_response.status_code = 500
        mock_requests.get.return_value = self.mock_response
        self.assertRaises(exceptions.ApiClientException,
                          self.session_manager.get, 'test_session_id')

    @mock.patch('freezerclient.v2.managers.sessions.requests')
    def test_get_none(self, mock_requests):
        self.mock_response.status_code = 404
        mock_requests.get.return_value = self.mock_response
        retval = self.session_manager.get('test_session_id')
        self.assertIsNone(retval)

    @mock.patch('freezerclient.v2.managers.sessions.requests')
    def test_list_ok(self, mock_requests):
        self.mock_response.status_code = 200
        session_list = [{'session_id_0': 'bomboloid'},
                        {'session_id_1': 'asdfasdf'}]
        self.mock_response.json.return_value = {'sessions': session_list}
        mock_requests.get.return_value = self.mock_response
        retval = self.session_manager.list()
        self.assertEqual(session_list, retval)

    @mock.patch('freezerclient.v2.managers.sessions.requests')
    def test_list_raise_ApiClientException_when_api_return_error_code(
            self, mock_requests):
        self.mock_response.status_code = 404
        session_list = [{'session_id_0': 'bomboloid'},
                        {'session_id_1': 'asdfasdf'}]
        self.mock_response.json.return_value = {'clients': session_list}
        mock_requests.get.return_value = self.mock_response
        self.assertRaises(exceptions.ApiClientException,
                          self.session_manager.list)

    @mock.patch('freezerclient.v2.managers.sessions.requests')
    def test_update_ok(self, mock_requests):
        self.mock_response.status_code = 200
        self.mock_response.json.return_value = {
            "patch": {"status": "bamboozled"},
            "version": 12,
            "session_id": "d454beec-1f3c-4d11-aa1a-404116a40502"
        }
        mock_requests.patch.return_value = self.mock_response
        retval = self.session_manager.update(
            'd454beec-1f3c-4d11-aa1a-404116a40502', {'status': 'bamboozled'})
        self.assertEqual(12, retval)

    @mock.patch('freezerclient.v2.managers.sessions.requests')
    def test_update_raise_ApiClientException_when_api_return_error_code(
            self, mock_requests):
        self.mock_response.json.return_value = {
            "patch": {"status": "bamboozled"},
            "version": 12,
            "session_id": "d454beec-1f3c-4d11-aa1a-404116a40502"
        }
        self.mock_response.status_code = 404
        self.mock_response.text = (
            '{"title": "Not Found","description":"No document found with ID '
            'd454beec-1f3c-4d11-aa1a-404116a40502x"}'
        )
        mock_requests.patch.return_value = self.mock_response
        self.assertRaises(exceptions.ApiClientException,
                          self.session_manager.update,
                          'd454beec-1f3c-4d11-aa1a-404116a40502',
                          {'status': 'bamboozled'})

    @mock.patch('freezerclient.v2.managers.sessions.requests')
    def test_add_job_uses_proper_endpoint(self, mock_requests):
        session_id, job_id = 'sessionqwerty1234', 'jobqwerty1234'
        self.mock_response.status_code = 204
        mock_requests.put.return_value = self.mock_response
        endpoint = '{0}{1}/jobs/{2}'.format(self.endpoint, session_id, job_id)

        retval = self.session_manager.add_job(session_id, job_id)

        self.assertIsNone(retval)
        mock_requests.put.assert_called_with(endpoint, headers=self.headers,
                                             verify=True)

    @mock.patch('freezerclient.v2.managers.sessions.requests')
    def test_add_job_raise_ApiClientException_when_api_return_error_code(
            self, mock_requests):
        session_id, job_id = 'sessionqwerty1234', 'jobqwerty1234'
        self.mock_response.status_code = 500
        mock_requests.put.return_value = self.mock_response
        self.assertRaises(exceptions.ApiClientException,
                          self.session_manager.add_job, session_id, job_id)

    @mock.patch('freezerclient.v2.managers.sessions.requests')
    def test_remove_job_uses_proper_endpoint(self, mock_requests):
        session_id, job_id = 'sessionqwerty1234', 'jobqwerty1234'
        self.mock_response.status_code = 204
        mock_requests.delete.return_value = self.mock_response
        endpoint = '{0}{1}/jobs/{2}'.format(self.endpoint, session_id, job_id)

        retval = self.session_manager.remove_job(session_id, job_id)

        self.assertIsNone(retval)
        mock_requests.delete.assert_called_with(endpoint, headers=self.headers,
                                                verify=True)

    @mock.patch('freezerclient.v2.managers.sessions.requests')
    def test_remove_job_raise_ApiClientException_when_api_return_error_code(
            self, mock_requests):
        session_id, job_id = 'sessionqwerty1234', 'jobqwerty1234'
        self.mock_response.status_code = 500
        mock_requests.delete.return_value = self.mock_response
        self.assertRaises(exceptions.ApiClientException,
                          self.session_manager.remove_job, session_id, job_id)

    @mock.patch('freezerclient.v2.managers.sessions.requests')
    def test_start_session_posts_proper_data(self, mock_requests):
        session_id, job_id, tag = 'sessionqwerty1234', 'jobqwerty1234', 23
        self.mock_response.status_code = 202
        self.mock_response.json.return_value = {'result': 'success',
                                                'session_tag': 24}
        mock_requests.post.return_value = self.mock_response
        # /v2/{project_id}/sessions/{sessions_id}/action
        endpoint = '{0}{1}/action'.format(self.endpoint, session_id)
        data = {"start": {"current_tag": 23, "job_id": "jobqwerty1234"}}
        retval = self.session_manager.start_session(session_id, job_id, tag)
        self.assertEqual({'result': 'success', 'session_tag': 24}, retval)

        args = mock_requests.post.call_args[0]
        kwargs = mock_requests.post.call_args[1]
        self.assertEqual(endpoint, args[0])
        self.assertEqual(data, json.loads(kwargs['data']))
        self.assertEqual(self.headers, kwargs['headers'])

    @mock.patch('freezerclient.v2.managers.sessions.requests')
    def test_start_session_raise_ApiClientException_when_api_return_error_code(
            self, mock_requests):
        session_id, job_id, tag = 'sessionqwerty1234', 'jobqwerty1234', 23
        self.mock_response.status_code = 500
        self.mock_response.json.return_value = {'result': 'success',
                                                'session_tag': 24}
        mock_requests.post.return_value = self.mock_response
        self.assertRaises(exceptions.ApiClientException,
                          self.session_manager.start_session,
                          session_id, job_id, tag)

    @mock.patch('freezerclient.v2.managers.sessions.requests')
    def test_end_session_posts_proper_data(self, mock_requests):
        session_id, job_id, tag = 'sessionqwerty1234', 'jobqwerty1234', 23
        self.mock_response.status_code = 202
        self.mock_response.json.return_value = {'result': 'success',
                                                'session_tag': 24}
        mock_requests.post.return_value = self.mock_response
        # /v2/{project_id}/sessions/{sessions_id}/action
        endpoint = '{0}{1}/action'.format(self.endpoint, session_id)
        data = {"end": {"current_tag": 23, "job_id": "jobqwerty1234",
                        "result": "fail"}}
        retval = self.session_manager.end_session(session_id, job_id, tag,
                                                  'fail')
        self.assertEqual({'result': 'success', 'session_tag': 24}, retval)

        args = mock_requests.post.call_args[0]
        kwargs = mock_requests.post.call_args[1]
        self.assertEqual(endpoint, args[0])
        self.assertEqual(data, json.loads(kwargs['data']))
        self.assertEqual(self.headers, kwargs['headers'])

    @mock.patch('freezerclient.v2.managers.sessions.requests')
    def test_end_session_raise_ApiClientException_when_api_return_error_code(
            self, mock_requests):
        session_id, job_id, tag = 'sessionqwerty1234', 'jobqwerty1234', 23
        self.mock_response.status_code = 500
        self.mock_response.json.return_value = {'result': 'success',
                                                'session_tag': 24}
        mock_requests.post.return_value = self.mock_response
        self.assertRaises(exceptions.ApiClientException,
                          self.session_manager.end_session,
                          session_id, job_id, tag, 'fail')



# (c) Copyright 2014-2016 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from cliff import command
from cliff import lister
from cliff import show

from freezerclient import exceptions
from freezerclient import utils


logging = logging.getLogger(__name__)


class ClientShow(show.ShowOne):
    """Show a single client"""
    def get_parser(self, prog_name):
        parser = super(ClientShow, self).get_parser(prog_name)
        parser.add_argument(dest='client_id',
                            help='ID of the client')
        return parser

    def take_action(self, parsed_args):
        client = self.app.client.clients.get(parsed_args.client_id)

        if not client:
            raise exceptions.ApiClientException('Client not found')

        column = (
            'Client ID',
            'Client UUID',
            'hostname',
            'description'
        )
        data = (
            client.get('client', {}).get('client_id'),
            client.get('client', {}).get('uuid'),
            client.get('client', {}).get('hostname'),
            client.get('client', {}).get('description', '')
        )

        return column, data


class ClientList(lister.Lister):
    """List of clients registered in the api"""
    def get_parser(self, prog_name):
        parser = super(ClientList, self).get_parser(prog_name)

        parser.add_argument(
            '--limit',
            dest='limit',
            default=100,
            help='Specify a limit for search query',
        )

        parser.add_argument(
            '--offset',
            dest='offset',
            default=0,
            help='',
        )

        parser.add_argument(
            '--search',
            dest='search',
            default='',
            help='Define a filter for the query',
        )
        return parser

    def take_action(self, parsed_args):
        search = utils.prepare_search(parsed_args.search)

        clients = self.app.client.clients.list(limit=parsed_args.limit,
                                               offset=parsed_args.offset,
                                               search=search)

        # Print empty table if no clients found
        if not clients:
            clients = [{}]

        columns = ('Client ID', 'uuid', 'hostname', 'description')
        data = ((
            client.get('client', {}).get('client_id', ''),
            client.get('client', {}).get('uuid', ''),
            client.get('client', {}).get('hostname', ''),
            client.get('client', {}).get('description', '')
        ) for client in clients)

        return columns, data


class ClientDelete(command.Command):
    """Delete a client from the api"""
    def get_parser(self, prog_name):
        parser = super(ClientDelete, self).get_parser(prog_name)
        parser.add_argument(dest='client_id',
                            help='ID of the client')
        return parser

    def take_action(self, parsed_args):
        self.app.client.clients.delete(parsed_args.client_id)
        logging.info('Client {0} deleted'.format(parsed_args.client_id))


class ClientRegister(command.Command):
    """Register a new client"""
    def get_parser(self, prog_name):
        parser = super(ClientRegister, self).get_parser(prog_name)
        parser.add_argument('--file',
                            dest='file',
                            required=True,
                            help='Path to json file with the client')
        return parser

    def take_action(self, parsed_args):
        client = utils.doc_from_json_file(parsed_args.file)
        try:
            client_id = self.app.client.clients.create(client)
        except Exception as err:
            raise exceptions.ApiClientException(err.message)
        else:
            logging.info("Client {0} registered".format(client_id))



# (c) Copyright 2014-2016 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import requests

from oslo_serialization import jsonutils as json

from freezerclient import exceptions
from freezerclient import utils


class BackupsManager(object):

    def __init__(self, client, verify=True):
        self.client = client
        self.endpoint = '{0}/v2/{1}/backups/'.format(
            self.client.endpoint, self.client.project_id)
        self.verify = verify

    @property
    def headers(self):
        return utils.create_headers_for_request(self.client.auth_token)

    def create(self, backup_metadata):
        r = requests.post(self.endpoint,
                          data=json.dumps(backup_metadata),
                          headers=self.headers,
                          verify=self.verify)
        if r.status_code != 201:
            raise exceptions.ApiClientException(r)
        backup_id = r.json()['backup_id']
        return backup_id

    def delete(self, backup_id):
        endpoint = self.endpoint + backup_id
        r = requests.delete(endpoint, headers=self.headers, verify=self.verify)
        if r.status_code != 204:
            raise exceptions.ApiClientException(r)

    def list(self, limit=10, offset=0, search=None):
        """
        Retrieves a list of backup infos

        :param limit: number of result to return (optional, default 10)
        :param offset: order of first document (optional, default 0)
        :param search: structured query (optional)
                       can contain:
                       * "time_before": timestamp
                       * "time_after": timestamp
                       Example:
                       { "time_before": 1428529956 }
        """
        data = json.dumps(search) if search else None
        query = {'limit': int(limit), 'offset': int(offset)}
        r = requests.get(self.endpoint, headers=self.headers,
                         params=query, data=data, verify=self.verify)
        if r.status_code != 200:
            raise exceptions.ApiClientException(r)

        return r.json()['backups']

    def get(self, backup_id):
        endpoint = self.endpoint + backup_id
        r = requests.get(endpoint, headers=self.headers, verify=self.verify)
        if r.status_code == 200:
            return r.json()
        if r.status_code == 404:
            return None
        raise exceptions.ApiClientException(r)



# (c) Copyright 2014-2016 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import logging
import os

from oslo_serialization import jsonutils as json
from oslo_utils import importutils

logging = logging.getLogger(__name__)


class Namespace(dict):
    """A dict subclass that exposes its items as attributes.

    Warning: Namespace instances do not have direct access to the
    dict methods.

    """

    def __init__(self, obj={}):
        super(Namespace, self).__init__(obj)

    def __dir__(self):
        return tuple(self)

    def __repr__(self):
        return "%s(%s)" % (type(self).__name__,
                           super(Namespace, self).__repr__())

    def __getattribute__(self, name):
        try:
            return self[name]
        except KeyError:
            # Return None in case the value doesn't exists
            # this is not an issue for the apiclient because it skips
            # None values
            return None

    def __setattr__(self, name, value):
        self[name] = value

    def __delattr__(self, name):
        del self[name]

    @classmethod
    def from_object(cls, obj, names=None):
        if names is None:
            names = dir(obj)
        ns = {name: getattr(obj, name) for name in names}
        return cls(ns)

    @classmethod
    def from_mapping(cls, ns, names=None):
        if names:
            ns = {name: ns[name] for name in names}
        return cls(ns)

    @classmethod
    def from_sequence(cls, seq, names=None):
        if names:
            seq = {name: val for name, val in seq if name in names}
        return cls(seq)

    @staticmethod
    def hasattr(ns, name):
        try:
            object.__getattribute__(ns, name)
        except AttributeError:
            return False
        return True

    @staticmethod
    def getattr(ns, name):
        return object.__getattribute__(ns, name)

    @staticmethod
    def setattr(ns, name, value):
        return object.__setattr__(ns, name, value)

    @staticmethod
    def delattr(ns, name):
        return object.__delattr__(ns, name)


class CachedProperty(object):
    def __init__(self, func):
        self.__doc__ = getattr(func, '__doc__')
        self.func = func

    def __get__(self, obj, cls):
        if obj is None:
            return self
        value = obj.__dict__[self.func.__name__] = self.func(obj)
        return value


def doc_from_json_file(path_to_file):
    """Build a json from a file in the file system
    :param path_to_file: path to file
    :return: in memory file in json format
    """
    with open(path_to_file, 'rb') as fd:
        try:
            return json.load(fd)
        except Exception as err:
            logging.error(err)
            raise Exception('Unable to load conf file. {0}'.format(err))


def create_headers_for_request(token):
    """Create a header dict to be passed to the api.

    :param token: token string coming from the api
    :return: a dict containing all the headers for a request
    """
    return {
        'X-Auth-Token': token,
        'Content-Type': 'application/json',
        'Accept': 'application/json'
    }


def prepare_search(search_term):
    """Return a search term that the api can use to query the db
    :param search_term: string
    :return: search dict
    """
    if search_term:
        return {"match": [{"_all": search_term}, ], }
    return {}


def check_api_version(api_version=None):
    """Check freezer version API to use
    1: not multi-tenant, useful for infrastructure
    2: multi-tenant, useful for backup as a service
    :return: str
    """
    if not api_version:
        freezer_api_version = os.environ.get('OS_BACKUP_API_VERSION', '2')
    else:
        freezer_api_version = api_version

    if freezer_api_version == '1':
        return '1'
    elif freezer_api_version == '2':
        return '2'
    else:
        raise Exception('Freezer API version not supported')


def get_client_class(api_version=None):
    """Return Client Class.
    Try to use provided api_version to import client class or
    Guess the api_version form env.
    Returns freezerclient.v{x}.client.Client
    :return: class
    """
    freezer_api_version = check_api_version(api_version)
    api_string = 'freezerclient.v{0}.client.Client'.format(freezer_api_version)
    return importutils.import_class(api_string)


def get_client_instance(kwargs={}, opts=None, api_version=None):
    """Get Freezerclient Instance.
    We will the provided auth dict to instantiate a client instance
    Returns freezerclient.v{x}.client.Client Object
    :return: Object
    """
    return get_client_class(api_version)(opts=opts, **kwargs)



# (c) Copyright 2018 ZTE Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import re
import sys

import fixtures
import io
import testtools
from testtools import matchers

from freezerclient import shell as openstack_shell


DEFAULT_USERNAME = 'username'
DEFAULT_PASSWORD = 'password'
DEFAULT_PROJECT_ID = 'tenant_id'
DEFAULT_PROJECT_NAME = 'tenant_name'
DEFAULT_AUTH_URL = 'http://127.0.0.1:5000/v2.0/'


class ShellTest(testtools.TestCase):

    FAKE_ENV = {
        'OS_USERNAME': DEFAULT_USERNAME,
        'OS_PASSWORD': DEFAULT_PASSWORD,
        'OS_TENANT_ID': DEFAULT_PROJECT_ID,
        'OS_TENANT_NAME': DEFAULT_PROJECT_NAME,
        'OS_PROJECT_ID': DEFAULT_PROJECT_ID,
        'OS_PROJECT_NAME': DEFAULT_PROJECT_NAME,
        'OS_AUTH_URL': DEFAULT_AUTH_URL,
    }

    # Patch os.environ to avoid required auth info.
    def setUp(self):
        super(ShellTest, self).setUp()
        for var in self.FAKE_ENV:
            self.useFixture(
                fixtures.EnvironmentVariable(
                    var, self.FAKE_ENV[var]))

    def shell(self, argstr, check=False, expected_val=0):
        # expected_val is the expected return value after executing
        # the command in FreezerShell
        orig = (sys.stdout, sys.stderr)
        clean_env = {}
        _old_env, os.environ = os.environ, clean_env.copy()
        try:
            sys.stdout = io.StringIO()
            sys.stderr = io.StringIO()
            _shell = openstack_shell.FreezerShell()
            _shell.run(argstr.split())
        except SystemExit:
            exc_type, exc_value, exc_traceback = sys.exc_info()
            self.assertEqual(expected_val, exc_value.code)
        finally:
            stdout = sys.stdout.getvalue()
            stderr = sys.stderr.getvalue()
            sys.stdout.close()
            sys.stderr.close()
            sys.stdout, sys.stderr = orig
            os.environ = _old_env
        return stdout, stderr, _shell.options

    def test_help(self):
        required = 'usage:'
        help_text, stderr, _ = self.shell('help')
        self.assertThat(
            help_text,
            matchers.MatchesRegex(required))

    def test_help_on_subcommand(self):
        required = [
            '.*?^usage: .* job-list']
        stdout, stderr, _ = self.shell('help job-list')
        for r in required:
            self.assertThat(
                stdout,
                matchers.MatchesRegex(r, re.DOTALL | re.MULTILINE))

    def test_help_command(self):
        required = 'usage:'
        help_text, stderr, _ = self.shell('help action-create')
        self.assertThat(
            help_text,
            matchers.MatchesRegex(required))

    def test_run_incomplete_command(self):
        cmd = 'job-create'
        stdout, stderr, _ = self.shell(cmd, check=True, expected_val=2)
        search_str = "run job-create: error"
        self.assertTrue(any(search_str in string for string
                            in stderr.split('\n')))

    def test_set_os_backup_api_version(self):
        cmd = (
            '--os-backup-api-version 1 job-list')
        stdout, stderr, options = self.shell(cmd)
        self.assertEqual("1", options.os_backup_api_version)

    def test_default_os_backup_api_version(self):
        cmd = 'help job-list'
        stdout, stderr, options = self.shell(cmd)
        self.assertEqual("2", options.os_backup_api_version)

    def test_set_os_username_password(self):
        cmd = (
            '--os-username caihui --os-password stack  job-list')
        stdout, stderr, options = self.shell(cmd)
        self.assertEqual("caihui", options.os_username)
        self.assertEqual("stack", options.os_password)

    def test_set_os_project_name_id(self):
        cmd = (
            '--os-project-id tecs0000000001 \
             --os-project-name tecs   job-list')
        stdout, stderr, options = self.shell(cmd)
        self.assertEqual("tecs0000000001", options.os_project_id)
        self.assertEqual("tecs", options.os_project_name)

    def test_set_os_auth_url(self):
        cmd = (
            '--os-auth-url http://127.0.0.1:5001 job-list')
        stdout, stderr, options = self.shell(cmd)
        self.assertEqual("http://127.0.0.1:5001", options.os_auth_url)



# (c) Copyright 2014-2016 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import requests

from oslo_serialization import jsonutils as json

from freezerclient import exceptions
from freezerclient import utils


class SessionManager(object):

    def __init__(self, client, verify=True):
        self.client = client
        self.endpoint = self.client.endpoint + '/v1/sessions/'
        self.verify = verify

    @property
    def headers(self):
        return utils.create_headers_for_request(self.client.auth_token)

    def create(self, doc, session_id=''):
        session_id = session_id or doc.get('session_id', '')
        endpoint = self.endpoint + session_id
        r = requests.post(endpoint,
                          data=json.dumps(doc),
                          headers=self.headers,
                          verify=self.verify)
        if r.status_code != 201:
            raise exceptions.ApiClientException(r)
        session_id = r.json()['session_id']
        return session_id

    def delete(self, session_id):
        endpoint = self.endpoint + session_id
        r = requests.delete(endpoint, headers=self.headers, verify=self.verify)
        if r.status_code != 204:
            raise exceptions.ApiClientException(r)

    def list_all(self, limit=10, offset=0, search=None):
        data = json.dumps(search) if search else None
        query = {'limit': int(limit), 'offset': int(offset)}
        r = requests.get(self.endpoint, headers=self.headers,
                         params=query, data=data, verify=self.verify)
        if r.status_code != 200:
            raise exceptions.ApiClientException(r)
        return r.json()['sessions']

    def list(self, limit=10, offset=0, search={}):
        new_search = search.copy()
        new_search['match'] = search.get('match', [])
        return self.list_all(limit, offset, new_search)

    def get(self, session_id):
        endpoint = self.endpoint + session_id
        r = requests.get(endpoint, headers=self.headers, verify=self.verify)
        if r.status_code == 200:
            return r.json()
        if r.status_code == 404:
            return None
        raise exceptions.ApiClientException(r)

    def update(self, session_id, update_doc):
        endpoint = self.endpoint + session_id
        r = requests.patch(endpoint,
                           headers=self.headers,
                           data=json.dumps(update_doc),
                           verify=self.verify)
        if r.status_code != 200:
            raise exceptions.ApiClientException(r)
        return r.json()['version']

    def add_job(self, session_id, job_id):
        # endpoint /v1/sessions/{sessions_id}/jobs/{job_id}
        endpoint = '{0}{1}/jobs/{2}'.format(self.endpoint, session_id, job_id)
        r = requests.put(endpoint,
                         headers=self.headers, verify=self.verify)
        if r.status_code != 204:
            raise exceptions.ApiClientException(r)
        return

    def remove_job(self, session_id, job_id):
        # endpoint /v1/sessions/{sessions_id}/jobs/{job_id}
        endpoint = '{0}{1}/jobs/{2}'.format(self.endpoint, session_id, job_id)
        retry = 5
        r = ''
        while retry:
            r = requests.delete(endpoint,
                                headers=self.headers, verify=self.verify)
            if r.status_code == 204:
                return
            retry -= 1
        raise exceptions.ApiClientException(r)

    def start_session(self, session_id, job_id, session_tag):
        """
        Informs the api that the client is starting the session
        identified by the session_id and request the session_tag
        to be incremented up to the requested value.
        The returned session_id could be:
         * current_tag + 1 if the session has started
         * > current_tag + 1 if the action had already been started
           by some other node and this node was out of sync

        :param session_id:
        :param job_id:
        :param session_tag: the new session_id
        :return: the response obj:
                 { result: string 'running' or 'error',
                  'session_tag': the new session_tag )
        """
        # endpoint /v1/sessions/{sessions_id}/action
        endpoint = '{0}{1}/action'.format(self.endpoint, session_id)
        doc = {"start": {
            "job_id": job_id,
            "current_tag": session_tag
        }}
        r = requests.post(endpoint,
                          headers=self.headers,
                          data=json.dumps(doc),
                          verify=self.verify)
        if r.status_code != 202:
            raise exceptions.ApiClientException(r)
        return r.json()

    def end_session(self, session_id, job_id, session_tag, result):
        """
        Informs the freezer service that the job has ended.
        Provides information about the job's result and the session tag

        :param session_id:
        :param job_id:
        :param session_tag:
        :param result:
        :return:
        """
        # endpoint /v1/sessions/{sessions_id}/action
        endpoint = '{0}{1}/action'.format(self.endpoint, session_id)
        doc = {"end": {
            "job_id": job_id,
            "current_tag": session_tag,
            "result": result
        }}
        r = requests.post(endpoint,
                          headers=self.headers,
                          data=json.dumps(doc),
                          verify=self.verify)
        if r.status_code != 202:
            raise exceptions.ApiClientException(r)
        return r.json()



# (c) Copyright 2014-2016 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import socket

from keystoneauth1 import loading as kaloading

from freezerclient import utils
from freezerclient.v2.managers import actions
from freezerclient.v2.managers import backups
from freezerclient.v2.managers import clients
from freezerclient.v2.managers import jobs
from freezerclient.v2.managers import sessions

FREEZER_SERVICE_TYPE = 'backup'


class Client(object):
    """Client for the OpenStack Disaster Recovery v2 API.
    """

    def __init__(self, token=None, username=None, password=None,
                 tenant_name=None, auth_url=None, session=None, endpoint=None,
                 endpoint_type=None, opts=None, project_name=None,
                 user_domain_name=None, user_domain_id=None,
                 project_domain_name=None, project_domain_id=None,
                 cert=None, cacert=None, insecure=False, project_id=None):
        """
        Initialize a new client for the Disaster Recovery v2 API.
        :param token: keystone token
        :param username: openstack username
        :param password: openstack password
        :param tenant_name: tenant
        :param auth_url: keystone-api endpoint
        :param session: keystone.Session
        :param endpoint: freezer-api endpoint
        :param endpoint_type: type of endpoint
        :param opts: a namespace to store all keystone data
        :param project_name: only for version 3
        :param tenant_id: only for version 2
        :param user_domain_name: only for version 3
        :param user_domain_id: only for version 3
        :param project_domain_name: only for version 3
        :param project_domain_id: only for version 3
        :param insecure: The verification arguments to pass to requests.
                       These are of the same form as requests expects,
                       so True or False to verify (or not) against system
                       certificates or a path to a bundle or CA certs to
                       check against or None for requests to
                       attempt to locate and use certificates. (optional,
                       defaults to True)
        :param cert: Path to cert
        :return: freezerclient.Client
        """

        self.project_id = project_id

        if opts is None:
            self.opts = utils.Namespace({})
            self.opts.os_token = token or None
            self.opts.os_username = username or None
            self.opts.os_password = password or None
            self.opts.os_tenant_name = tenant_name or None
            self.opts.os_auth_url = auth_url or None
            self.opts.os_backup_url = endpoint or None
            self.opts.os_endpoint_type = endpoint_type or None
            self.opts.os_project_name = project_name or None
            self.opts.os_project_id = project_id or None
            self.opts.os_user_domain_name = user_domain_name or None
            self.opts.os_user_domain_id = user_domain_id or None
            self.opts.os_project_domain_name = project_domain_name or None
            self.opts.os_project_domain_id = project_domain_id or None
            self.opts.os_cacert = cacert or None
            self.opts.insecure = insecure
            self.opts.cert = cert
        else:
            self.opts = opts

        self.cert = cert
        self.cacert = cacert or self.opts.os_cacert
        self._session = session
        verify = self.opts.os_cacert
        if self.opts.insecure:
            verify = False

        self.validate()
        self.project_id = project_id or self.get_project_id

        self.jobs = jobs.JobManager(self, verify=verify)
        self.clients = clients.ClientManager(self, verify=verify)
        self.backups = backups.BackupsManager(self, verify=verify)
        self.sessions = sessions.SessionManager(self, verify=verify)
        self.actions = actions.ActionManager(self, verify=verify)

    @utils.CachedProperty
    def session(self):
        if self._session:
            return self._session
        auth_type = 'password'
        auth_kwargs = {
            'auth_url': self.opts.os_auth_url,
            'project_id': self.opts.os_project_id,
            'project_name': self.opts.os_project_name,
            'project_domain_id': self.opts.os_project_domain_id,
            'project_domain_name': self.opts.os_project_domain_name,
        }
        if self.opts.os_username and self.opts.os_password:
            auth_kwargs.update({
                'username': self.opts.os_username,
                'password': self.opts.os_password,
                'tenant_name': self.opts.os_tenant_name,
                'user_domain_id': self.opts.os_user_domain_id,
                'user_domain_name': self.opts.os_user_domain_name,
            })
        elif self.opts.os_token:
            auth_type = 'token'
            auth_kwargs.update({
                'token': self.opts.os_token,
            })
        loader = kaloading.get_plugin_loader(auth_type)
        auth_plugin = loader.load_from_options(**auth_kwargs)
        # Let keystoneauth do the necessary parameter conversions
        session = kaloading.session.Session().load_from_options(
            auth=auth_plugin, insecure=self.opts.insecure, cacert=self.cacert,
            cert=self.cert)
        return session

    @utils.CachedProperty
    def endpoint(self):
        if self.opts.os_backup_url:
            return self.opts.os_backup_url
        else:
            auth_ref = self.session.auth.get_auth_ref(self.session)
            endpoint = auth_ref.service_catalog.url_for(
                service_type=FREEZER_SERVICE_TYPE,
                interface=self.opts.os_endpoint_type,
            )
        return endpoint

    @property
    def auth_token(self):
        return self.session.get_token()

    @property
    def get_project_id(self):
        return self.session.get_project_id()

    @utils.CachedProperty
    def client_id(self):
        return '{0}_{1}'.format(self.session.get_project_id(),
                                socket.gethostname())

    def validate(self):
        """Validate that the client objects gets created correctly.
        :return: bool
        """
        if not self._session and self.opts.os_auth_url is None:
            raise Exception('OS_AUTH_URL should be provided.')



# (c) Copyright 2014-2016 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import pprint

from cliff import command
from cliff import lister
from cliff import show

from freezerclient import exceptions
from freezerclient import utils


logging = logging.getLogger(__name__)


class JobShow(show.ShowOne):
    """Show a single job"""
    def get_parser(self, prog_name):
        parser = super(JobShow, self).get_parser(prog_name)
        parser.add_argument(dest='job_id',
                            help='ID of the job')
        return parser

    def take_action(self, parsed_args):
        job = self.app.client.jobs.get(parsed_args.job_id)

        if not job:
            raise exceptions.ApiClientException('Job not found')

        column = (
            'Job ID',
            'Client ID',
            'User ID',
            'Session ID',
            'Description',
            'Actions',
            'Start Date',
            'End Date',
            'Interval',
            'Status',
            'Result',
            'Current pid',
            'Event'
        )
        data = (
            job.get('job_id'),
            job.get('client_id'),
            job.get('user_id'),
            job.get('session_id', ''),
            job.get('description'),
            pprint.pformat(job.get('job_actions')),
            job.get('job_schedule', {}).get('schedule_start_date', ''),
            job.get('job_schedule', {}).get('schedule_end_date', ''),
            job.get('job_schedule', {}).get('schedule_interval', ''),
            job.get('job_schedule', {}).get('status', ''),
            job.get('job_schedule', {}).get('result', ''),
            job.get('job_schedule', {}).get('current_pid', ''),
            job.get('job_schedule', {}).get('event', ''),
        )
        return column, data


class JobList(lister.Lister):
    """List all the jobs for your user"""
    def get_parser(self, prog_name):
        parser = super(JobList, self).get_parser(prog_name)

        parser.add_argument(
            '--limit',
            dest='limit',
            default=100,
            help='Specify a limit for search query',
        )

        parser.add_argument(
            '--offset',
            dest='offset',
            default=0,
            help='',
        )

        parser.add_argument(
            '--search',
            dest='search',
            default={},
            help='Define a filter for the query',
        )

        parser.add_argument(
            '--client', '-C',
            dest='client_id',
            default='',
            help='Get jobs for a specific client',
        )
        return parser

    def take_action(self, parsed_args):

        search = utils.prepare_search(parsed_args.search)

        if parsed_args.client_id:
            jobs = self.app.client.jobs.list(
                limit=parsed_args.limit,
                offset=parsed_args.offset,
                search=search,
                client_id=parsed_args.client_id
            )
        else:
            jobs = self.app.client.jobs.list_all(
                limit=parsed_args.limit,
                offset=parsed_args.offset,
                search=search
            )

        columns = ('Job ID', 'Description', '# Actions', 'Result', 'Status',
                   'Event', 'Session ID')

        # Print empty table if no jobs found
        if not jobs:
            jobs = [{}]
            data = ((job.get('job_id', ''),
                     job.get('description', ''),
                     job.get('job_actions', ''),
                     job.get('job_schedule', {}).get('result', ''),
                     job.get('job_schedule', {}).get('status', ''),
                     job.get('job_schedule', {}).get('event', ''),
                     job.get('session_id', '')
                     ) for job in jobs)
        else:
            data = ((job.get('job_id'),
                     job.get('description'),
                     len(job.get('job_actions', [])),
                     job.get('job_schedule', {}).get('result', ''),
                     job.get('job_schedule', {}).get('status', ''),
                     job.get('job_schedule', {}).get('event', ''),
                     job.get('session_id', '')
                     ) for job in jobs)

        return columns, data


class JobGet(command.Command):
    """Download a job as a json file"""
    def get_parser(self, prog_name):
        parser = super(JobGet, self).get_parser(prog_name)
        parser.add_argument(dest='job_id',
                            help='ID of the job')

        parser.add_argument('--no-format',
                            dest='no_format',
                            default=False,
                            action='store_true',
                            help='Return a job in json without pretty print')
        return parser

    def take_action(self, parsed_args):
        job = self.app.client.jobs.get(parsed_args.job_id)

        if not job:
            raise exceptions.ApiClientException('Job not found')

        if parsed_args.no_format:
            print(job)
        else:
            pprint.pprint(job)


class JobDelete(command.Command):
    """Delete a job from the api"""
    def get_parser(self, prog_name):
        parser = super(JobDelete, self).get_parser(prog_name)
        parser.add_argument(dest='job_id',
                            help='ID of the job')
        return parser

    def take_action(self, parsed_args):
        self.app.client.jobs.delete(parsed_args.job_id)
        logging.info('Job {0} deleted'.format(parsed_args.job_id))


class JobCreate(command.Command):
    """Create a new job from a file"""
    def get_parser(self, prog_name):
        parser = super(JobCreate, self).get_parser(prog_name)
        parser.add_argument(
            '--file',
            dest='file',
            required=True,
            help='Path to json file with the job')

        parser.add_argument(
            '--client', '-C',
            dest='client_id',
            required=True,
            help='Select a client for this job',
        )

        return parser

    def take_action(self, parsed_args):
        job = utils.doc_from_json_file(parsed_args.file)
        job['client_id'] = parsed_args.client_id
        job_id = self.app.client.jobs.create(job)
        logging.info('Job {0} created'.format(job_id))


class JobStart(command.Command):
    """Send a start signal for a job"""
    def get_parser(self, prog_name):
        parser = super(JobStart, self).get_parser(prog_name)
        parser.add_argument(dest='job_id',
                            help='ID of the job')
        return parser

    def take_action(self, parsed_args):
        self.app.client.jobs.start_job(parsed_args.job_id)
        logging.info("Start request sent "
                     "for job {0}".format(parsed_args.job_id))


class JobStop(command.Command):
    """Send a stop signal for a job"""
    def get_parser(self, prog_name):
        parser = super(JobStop, self).get_parser(prog_name)
        parser.add_argument(dest='job_id',
                            help='ID of the job')
        return parser

    def take_action(self, parsed_args):
        self.app.client.jobs.stop_job(parsed_args.job_id)
        logging.info("Stop request sent "
                     "for job {0}".format(parsed_args.job_id))


class JobAbort(command.Command):
    """Abort a running job"""
    def get_parser(self, prog_name):
        parser = super(JobAbort, self).get_parser(prog_name)
        parser.add_argument(dest='job_id',
                            help='ID of the job')
        return parser

    def take_action(self, parsed_args):
        self.app.client.jobs.abort_job(parsed_args.job_id)
        logging.info("Abort request sent "
                     "for job {0}".format(parsed_args.job_id))


class JobUpdate(command.Command):
    """Update a job from a file"""
    def get_parser(self, prog_name):
        parser = super(JobUpdate, self).get_parser(prog_name)
        parser.add_argument(dest='job_id',
                            help='ID of the job')

        parser.add_argument(dest='file',
                            help='Path to json file with the job')
        return parser

    def take_action(self, parsed_args):
        job = utils.doc_from_json_file(parsed_args.file)
        self.app.client.jobs.update(parsed_args.job_id, job)
        logging.info('Job {0} updated'.format(parsed_args.job_id))



# (c) Copyright 2014-2016 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import pprint

from cliff import command
from cliff import lister
from cliff import show

from freezerclient import exceptions
from freezerclient import utils


logging = logging.getLogger(__name__)


class SessionShow(show.ShowOne):
    """Show a single session"""
    def get_parser(self, prog_name):
        parser = super(SessionShow, self).get_parser(prog_name)
        parser.add_argument(dest='session_id',
                            help='ID of the session')
        return parser

    def take_action(self, parsed_args):
        session = self.app.client.sessions.get(parsed_args.session_id)

        if not session:
            raise exceptions.ApiClientException('Session not found')

        column = (
            'Session ID',
            'Session tag',
            'Description',
            'Status',
            'Result',
            'Jobs',
            'Hold off',
            'Schedule',
            'Last start',
            'Time start',
            'Time end',
            'User id'
        )

        data = (
            session.get('session_id'),
            session.get('session_tag'),
            session.get('description'),
            session.get('status'),
            session.get('result'),
            pprint.pformat(session.get('jobs')),
            session.get('hold_off'),
            pprint.pformat(session.get('schedule')),
            session.get('last_start'),
            session.get('time_start'),
            session.get('time_end'),
            session.get('user_id'),
        )
        return column, data


class SessionList(lister.Lister):
    """List all the sessions for your user"""
    def get_parser(self, prog_name):
        parser = super(SessionList, self).get_parser(prog_name)

        parser.add_argument(
            '--limit',
            dest='limit',
            default=100,
            help='Specify a limit for search query',
        )

        parser.add_argument(
            '--offset',
            dest='offset',
            default=0,
            help='',
        )

        parser.add_argument(
            '--search',
            dest='search',
            default='',
            help='Define a filter for the query',
        )
        return parser

    def take_action(self, parsed_args):
        search = utils.prepare_search(parsed_args.search)
        sessions = self.app.client.sessions.list_all(
            limit=parsed_args.limit,
            offset=parsed_args.offset,
            search=search
        )

        # Print empty table if no sessions found
        if not sessions:
            sessions = [{}]

        columns = ('Session ID', 'Description', 'Status',
                   'Result', '# Jobs')
        data = ((
            session.get('session_id', ''),
            session.get('description', ''),
            session.get('status', ''),
            session.get('result', ''),
            len(session.get('jobs', [])) if session.get(
                'session_id') else '',
        ) for session in sessions)

        return columns, data


class SessionCreate(command.Command):
    """Create a session from a file"""
    def get_parser(self, prog_name):
        parser = super(SessionCreate, self).get_parser(prog_name)
        parser.add_argument('--file',
                            dest='file',
                            required=True,
                            help='Path to json file with the job')
        return parser

    def take_action(self, parsed_args):
        session = utils.doc_from_json_file(parsed_args.file)
        session_id = self.app.client.sessions.create(session)
        logging.info('Session {0} created'.format(session_id))


class SessionDelete(command.Command):
    """Delete a session"""
    def get_parser(self, prog_name):
        parser = super(SessionDelete, self).get_parser(prog_name)
        parser.add_argument(dest='session_id',
                            help='ID of the session')
        return parser

    def take_action(self, parsed_args):
        session = self.app.client.sessions.get(parsed_args.session_id)
        if not session:
            logging.info('Unable to delete specified session.')
            raise exceptions.ApiClientException('Session not found')

        self.app.client.sessions.delete(parsed_args.session_id)
        logging.info('Session {0} deleted'.format(parsed_args.session_id))


class SessionAddJob(command.Command):
    """Add a job to a session"""
    def get_parser(self, prog_name):
        parser = super(SessionAddJob, self).get_parser(prog_name)
        parser.add_argument('--session-id',
                            dest='session_id',
                            required=True,
                            help='ID of the session')
        parser.add_argument('--job-id',
                            dest='job_id',
                            required=True,
                            help='ID of the job to add')
        return parser

    def take_action(self, parsed_args):
        self.app.client.sessions.add_job(parsed_args.session_id,
                                         parsed_args.job_id)
        logging.info('Job {0} added correctly to session {1}'.format(
            parsed_args.job_id, parsed_args.session_id))


class SessionRemoveJob(command.Command):
    """Remove a job from a session"""
    def get_parser(self, prog_name):
        parser = super(SessionRemoveJob, self).get_parser(prog_name)
        parser.add_argument('--session-id',
                            dest='session_id',
                            required=True,
                            help='ID of the session')
        parser.add_argument('--job-id',
                            dest='job_id',
                            required=True,
                            help='ID of the job to add')
        return parser

    def take_action(self, parsed_args):
        try:
            self.app.client.sessions.remove_job(parsed_args.session_id,
                                                parsed_args.job_id)
        except Exception as error:
            # there is an error coming from the api when a job is removed
            # with the following text:
            # Additional properties are not allowed
            # ('job_event' was unexpected)
            # but in reality the job gets removed correctly.
            if 'Additional properties are not allowed' in error.message:
                pass
            else:
                raise exceptions.ApiClientException(error.message)
        else:
            logging.info('Job {0} removed correctly from session {1}'.format(
                parsed_args.job_id, parsed_args.session_id))


class SessionUpdate(command.Command):
    """Update a session from a file"""
    def get_parser(self, prog_name):
        parser = super(SessionUpdate, self).get_parser(prog_name)
        parser.add_argument(dest='session_id',
                            help='ID of the session')

        parser.add_argument(dest='file',
                            help='Path to json file with the session')
        return parser

    def take_action(self, parsed_args):
        session = utils.doc_from_json_file(parsed_args.file)
        self.app.client.sessions.update(parsed_args.session_id, session)
        logging.info('Session {0} updated'.format(parsed_args.session_id))


class SessionStart(command.Command):
    """Start a session"""
    def get_parser(self, prog_name):
        parser = super(SessionStart, self).get_parser(prog_name)
        parser.add_argument('--session-id',
                            dest='session_id',
                            required=True,
                            help='ID of the session')
        parser.add_argument('--job-id',
                            dest='job_id',
                            required=True,
                            help='ID of the job')
        parser.add_argument('--job-tag',
                            dest='job_tag',
                            required=True,
                            help='Job tag value')

        return parser

    def take_action(self, parsed_args):
        session = self.app.client.sessions.get(parsed_args.session_id)
        if not session:
            logging.info('Unable to start specified session.')
            raise exceptions.ApiClientException('Session not found')

        self.app.client.sessions.start_session(
            parsed_args.session_id,
            parsed_args.job_id,
            parsed_args.job_tag
        )
        logging.info('Session {0} start requested'.format(
            parsed_args.session_id))



# (c) Copyright 2014-2016 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_serialization import jsonutils as json


class ApiClientException(Exception):
    @staticmethod
    def get_message_from_api_response(r):
        """
        returns a message based on information from a api-formatted response
        if available, otherwise None
        api-formatted response should be:
            {
                title: "error title string",
                description: "error description string"
            }

        :param r: response object
        :return: string with error message or None if it fails
        """
        try:
            body = json.loads(r.text)
            message = "[*] Error {0}: {1}".format(
                r.status_code,
                body['description'])
        except Exception:
            message = None
        return message

    @staticmethod
    def get_message_from_response(r):
        """
        composes the error message using information eventually present
        in the response (http error code and the http response body)

        :param r: response object
        :return: string with error message or None if it fails
        """
        try:
            message = "[*] Error {0}: {1}".format(
                r.status_code,
                r.text)
        except Exception:
            message = None
        return message

    def __init__(self, r):
        message = self.get_message_from_api_response(r) or \
            self.get_message_from_response(r) or \
            str(r)
        try:
            self.status_code = r.status_code
        except Exception:
            self.status_code = None
        super(ApiClientException, self).__init__(message)



# (c) Copyright 2018 ZTE Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from freezerclient import utils
import os


def Client(version=None, endpoint=None, username=None, password=None,
           project_name=None, auth_url=None, project_id=None, token=None,
           cacert=None, project_domain_name=None, user_domain_id=None,
           user_domain_name=None, **kwargs):
    """Initialize client object based on given version.

    HOW-TO:
    The simplest way to create a client instance is initialization with your
    credentials::

        >>> from freezerclient import client
        >>> freezer = client.Client('2',username='admin',password='stack')

    Here ``VERSION`` is freezer API Version, you can use ``1``(v1) or ``2``
    (v2),default is API v2.

    Alternatively, you can create a client instance using the keystoneauth
    session API. See "The freezerclient Python API" page at
    python-freezerclient's doc.
    """
    if endpoint:
        kwargs["endpoint"] = endpoint
    else:
        kwargs["endpoint"] = os.environ.get('OS_BACKUP_URL')

    if username:
        kwargs["username"] = username
    else:
        kwargs["username"] = os.environ.get('OS_USERNAME')

    if password:
        kwargs["password"] = password
    else:
        kwargs["password"] = os.environ.get('OS_PASSWORD')

    if project_name:
        kwargs["project_name"] = project_name
    else:
        kwargs["project_name"] = os.environ.get('OS_PROJECT_NAME')

    if auth_url:
        kwargs["auth_url"] = auth_url
    else:
        kwargs["auth_url"] = os.environ.get('OS_AUTH_URL')

    if token:
        kwargs["token"] = token
    else:
        kwargs["token"] = os.environ.get('OS_TOKEN')

    kwargs["project_domain_name"] = os.environ.get('OS_PROJECT_DOMAIN_NAME')
    if project_domain_name:
        kwargs["project_domain_name"] = project_domain_name

    if user_domain_name:
        kwargs["user_domain_name"] = user_domain_name
    else:
        kwargs["user_domain_name"] = os.environ.get('OS_USER_DOMAIN_NAME')

    if user_domain_id:
        kwargs["user_domain_id"] = user_domain_id
    else:
        kwargs["user_domain_id"] = os.environ.get('OS_USER_DOMAIN_ID')

    if project_id:
        kwargs["project_id"] = project_id
    else:
        kwargs["project_id"] = os.environ.get('OS_PROJECT_ID')

    kwargs["cacert"] = cacert or os.environ.get('OS_CACERT')
    kwargs["endpoint_type"] = os.environ.get('OS_ENDPOINT_TYPE') or 'public'

    if not version:
        version = os.environ.get('OS_BACKUP_API_VERSION', '2')

    client_class = utils.get_client_class(version)

    return client_class(**kwargs)
